{+-----------------------------------------------------------------------------+
 | Created:     1998-12-16
 | Last change: 1999-09-24
 | Author:      Primoz Gabrijelcic
 | Description: One unit to bind them - common Register for all mwEdit
 |              components
 | Version:     0.19
 | Copyright (c) 1998 Primoz Gabrijelcic
 | No rights reserved.
 |
 | Thanks to: Brad Stowers
 |
 | Version history: see version.rtf
 |
 +----------------------------------------------------------------------------+}

unit mwEditReg;

interface

procedure Register;

implementation

{begin}                                                                         //bds 1998-12-16
uses
  DsgnIntf, Dialogs, Forms, Graphics,
  mwKeyCmds, Classes,                                                           //bds 1998-12-23
  mwKeyCmdsEditor,                                                              //bds 1998-12-29
  mwCustomEdit, mwPasSyn, DcjCppSyn, DcjPerlSyn, mwGeneralSyn, cbHPSyn,
  DcjJavaSyn,                                                                   //mt 1999-01-02
  cwCACSyn,                                                                     //gp 1999-01-10
  wmSQLSyn,                                                                     //gp 1999-02-28
  hkHTMLSyn,                                                                    //gp 1999-05-05
  hkAWKSyn,                                                                     //gp 1999-05-05
  siTclTksyn,                                                                   //si 1999-05-02
  lbVBSSyn,                                                                     //gp 1999-05-06
  odPySyn, odPythonBehaviour,                                                   //od 1999-03-28
  mkGalaxySyn,                                                                  //gp 1999-06-10
  mwRTFExport,                                                                  //gp 1999-06-10
  mwHTMLExport,                                                                 //gp 1999-06-10
  DBmwEdit,                                                                     //mh 1999-09-24
  dmBatSyn,                                                                     //mh 1999-09-24
  dmDfmSyn,                                                                     //mh 1999-09-24
  nhAsmSyn,                                                                     //mh 1999-09-25
  mwCompletionProposal;                                                         //mh 1999-09-25

type
  TmwEditorFontProperty = class(TFontProperty)
  public
    procedure Edit; override;
  end;

procedure TmwEditorFontProperty.Edit;
const
  { context ids for the Font editor }
  hcDFontEditor = 25000;
var
  FontDialog: TFontDialog;
begin
  FontDialog := TFontDialog.Create(Application);
  try
    FontDialog.Font := TFont(GetOrdValue);
    FontDialog.HelpContext := hcDFontEditor;
    FontDialog.Options := FontDialog.Options + [fdShowHelp, fdForceFontExist,
       fdFixedPitchOnly];
    if FontDialog.Execute then
      SetOrdValue(Longint(FontDialog.Font));
  finally
    FontDialog.Free;
  end;
end;
{end}                                                                           //bds 1998-12-16

{begin}                                                                         //bds 1998-12-23
type
  TmwEditorCommandProperty = class(TIntegerProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
  end;

{ TmwEditorCommandProperty }

procedure TmwEditorCommandProperty.Edit;
begin
  ShowMessage('I''m thinking that this will show a dialog that has a list'#13#10+
     'of all editor commands and a description of them to choose from.');
end;

function TmwEditorCommandProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paDialog, paValueList, paRevertable];
end;

function TmwEditorCommandProperty.GetValue: string;
begin
  Result := EditorCommandToCodeString(TmwEditorCommand(GetOrdValue));
end;

procedure TmwEditorCommandProperty.GetValues(Proc: TGetStrProc);
begin
  GetEditorCommandValues(Proc);
end;

procedure TmwEditorCommandProperty.SetValue(const Value: string);
var
  NewValue: longint;                                                            //bds 1998-12-29
begin
  if IdentToEditorCommand(Value, NewValue) then
    SetOrdValue(NewValue)
  else
    inherited SetValue(Value);
end;

{end}                                                                           //bds 1998-12-23

procedure Register;
begin
  mwCustomEdit.Register;
  RegisterPropertyEditor(TypeInfo(TFont), TmwCustomEdit,                        //bds 1998-12-16
     'Font', TmwEditorFontProperty);
  RegisterPropertyEditor(TypeInfo(TmwEditorCommand), NIL,                       //bds 1998-12-23
     'Command', TmwEditorCommandProperty);
  RegisterPropertyEditor(TypeInfo(TmwKeystrokes), NIL, '',                      //bds 1998-12-29
     TmwKeyStrokesProperty);
  mwPasSyn.Register;
  DcjCppSyn.Register;
  DcjPerlSyn.Register;                                                          //gp 1998-12-30
  mwGeneralSyn.Register;                                                        //gp 1998-12-30
  cbHPSyn.Register;                                                             //gp 1998-12-30
  DcjJavaSyn.Register;                                                          //mt 1999-01-02
  cwCACSyn.Register;                                                            //gp 1999-01-10
  wmSQLSyn.Register;                                                            //gp 1999-02-28
  siTclTksyn.Register;                                                          //si 199-05-02
  hkHTMLSyn.Register;                                                           //gp 1999-05-05
  hkAWKSyn.Register;                                                            //gp 1999-05-05
  lbVBSSyn.Register;                                                            //gp 1999-05-06
  odPySyn.Register;                                                             //gp 1999-05-06
  odPythonBehaviour.Register;                                                   //gp 1999-05-06
  mkGalaxySyn.Register;                                                         //gp 1999-06-10
  mwRTFExport.Register;                                                         //gp 1999-06-10
  mwHTMLExport.Register;                                                        //gp 1999-06-10
  DBmwEdit.Register;                                                            //mh 1999-09-24
  dmBatSyn.Register;                                                            //mh 1999-09-24
  dmDfmSyn.Register;                                                            //mh 1999-09-24
  nhAsmSyn.Register;                                                            //mh 1999-09-25
  mwCompletionProposal.Register;                                                //mh 1999-09-25
end; { Register }

(* This moved to mwKeyCmds.pas because it has to be there under D3.             //bds 1998-12-29
{begin}                                                                         //bds 1998-12-23
initialization
  RegisterIntegerConsts(TypeInfo(TmwEditorCommand), IdentToEditorCommand,
     EditorCommandToIdent);
{end}                                                                           //bds 1998-12-23
*)                                                                              //bds 1998-12-29
end.
